//+------------------------------------------------------------------+
//| Stoch Diff 2 Pair EAsq.mq4
//+------------------------------------------------------------------+

#property copyright "roundrock"
#property link      ""

/* This EA trades the "Hedge and Correlation Strategy" thread at FF:
 * http://www.forexfactory.com/showthread.php?t=160912
 * 
 * This EA was initially posted by "roundrock" here:
 * http://www.forexfactory.com/showthread.php?p=4351329#post4351329
 *
 * Squalou main mods:
 * - added optional "ProfitTarget" input: will close both trades if the combined profit reaches this value (including swap and commission)
 *   (value in your account's currency: 10.0 means 10USD is your account is in USD, or 10EUR if EUR account);
 * - visual cosmetic changes:
 *   - trades opening is marked with a blue vertical line on the chart;
 *   - trades closure is marked with a red vertical line on the chart;
 *   - the Comment area displays various useful information:
 *     - EA start time and uptime;
 *     - Stoch values for both pairs, and Difference;
 *     - currently or last opened trades: pairs, direction, current individual profit/loss in account's currency units,
 *     - combined profit/loss, and MAX DRAWDOWN -- this will be helpful to determine inputs tuning;
 *     - last trades closure time
 * - some "cosmetic" code changes (inputs names, tabs, etc)
 * 
 */

#include <stderror.mqh>
#include <stdlib.mqh>

extern string Pair1 = "EURUSD";
extern string Pair2 = "USDCHF";
extern string OpenTradeStoch_notes = "--- Open Trade if stoch difference is above this number ---";
extern double OpenTradeStoch = 80;
extern string CloseTradesStoch_notes = "--- Close Trade if stoch difference is below this number ---";
extern double CloseTradesStoch = 20;
extern string ProfitTarget_notes = "--- Close Trade if Profit target reached ---";
extern double ProfitTarget = 10.00;
extern int K = 100;
extern int D = 1;
extern int S = 1;
extern int Slippage = 3;
extern double LotSize = 0.1;
extern int MagicNumber = 55555;


int digits = 4;
double pip = 0;
int barsTotal = 0;
string ccy;
int tf;
double pnt;
string m = "";

datetime time_EA_started;
string buyTradeStatus,sellTradeStatus,combined_PL,orderclosetime;
bool TradeExist;
double current_UPL, max_DD;

//+------------------------------------------------------------------+
int init()
//+------------------------------------------------------------------+
{
  m = StringSubstr(Symbol(), 6, StringLen(Symbol()) - 6);
  Pair2 = Pair2 + m;
  if (Pair1 == "") Pair1 = Symbol();
  else Pair1 = Pair1 + m;

  digits = MarketInfo(Symbol(), MODE_DIGITS);

  if (digits < 4) pip = 0.01;
  else pip = 0.0001;

  digits = MarketInfo(Symbol(), MODE_DIGITS);

  time_EA_started = TimeCurrent();

  return (0);
}

//+------------------------------------------------------------------+
int deinit()
//+------------------------------------------------------------------+
{
  return (0);
}

//+------------------------------------------------------------------+
int start()
//+------------------------------------------------------------------+
{
  static string last_comment = "";
  double stoch1 = iStochastic(Pair1, 0, K, D, S, 0, 0, 0, 0);
  double stoch2 = iStochastic(Pair2, 0, K, D, S, 0, 0, 0, 0);

  if (CheckTradeExist(Pair1,Pair2) || CheckTradeExist(Pair2,Pair1)) {

    // both trades exist: check for closing both
    if ( ( CloseTradesStoch>0  && MathAbs(stoch1 - stoch2) < CloseTradesStoch ) // Stoch Diff for closing reached
      || ( ProfitTarget>0 && current_UPL>=ProfitTarget )) // combined Profit target reached
    {
      // close existing trades
      last_comment = TimeToStr(TimeCurrent(),TIME_DATE|TIME_SECONDS)+ "Closing Trades Now"+"\n";
      CloseTrades();
      orderclosetime = "Last Orders Closed at "+TimeToStr(TimeCurrent(),TIME_DATE|TIME_SECONDS);
    }

  } else {

    // no trade exist: check for open
    if (MathAbs(stoch1 - stoch2) > OpenTradeStoch)
    {
      // ready to open trade.
      // check if same trade exists
      string buySym = "",sellSym = "";
      if (stoch1 < stoch2) {
        buySym = Pair1; 
        sellSym = Pair2;
      } else {
        buySym = Pair2;
        sellSym = Pair1;
      }

      Print("Ready to place orders. Stoch1 " + DoubleToStr(stoch1, 2) + " stoch2" + DoubleToStr(stoch2, 2) + " Difference " + DoubleToStr(MathAbs(stoch1 - stoch2), 2));
      // now place new trade         
      last_comment = TimeToStr(TimeCurrent(),TIME_DATE|TIME_SECONDS)+"Placing Buy("+Pair1+") + Sell("+Pair2+")"+"\n";
      placeOrders(buySym,sellSym);
    }
  }

  Comment(WindowExpertName()+" started on "+TimeToStr(time_EA_started)+" (uptime="+DoubleToStr((TimeCurrent()-time_EA_started)/60,0)+"min)\n"
      + last_comment
      +"Stoch("+Pair1+")="+DoubleToStr(stoch1,2)+"\n"
      +"Stoch("+Pair2+")="+DoubleToStr(stoch2,2)+"\n"
      +"Diff="+DoubleToStr(MathAbs(stoch1-stoch2),2)+"\n"
      +buyTradeStatus+"\n"
      +sellTradeStatus+"\n"
      +combined_PL+", max DD="+DoubleToStr(max_DD,2)+"\n"
      +orderclosetime+"\n"
  );

  return (0);
}

//+------------------------------------------------------------------+
bool CheckTradeExist(string buySym,string sellSym)
//+------------------------------------------------------------------+
{
  TradeExist = false;
  current_UPL = 0; // P/L of open orders
  for (int i = 0; i < OrdersTotal(); i++)
  {
    OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
    if (OrderMagicNumber() == MagicNumber)
    {
      // Print(" order type "+OrderType() +" ticket "+ OrderTicket());
      if (OrderType() == OP_BUY && OrderSymbol() == buySym)
      {
        current_UPL += OrderProfit()+OrderSwap()+OrderCommission();
        buyTradeStatus = "Last BUY "+OrderSymbol()+" Opened at "+TimeToStr(OrderOpenTime(),TIME_DATE|TIME_SECONDS)+" P/L = "+DoubleToStr(OrderProfit()+OrderSwap()+OrderCommission(),2);
        TradeExist = true;
      }
      if (OrderType() == OP_SELL && OrderSymbol() == sellSym)
      {
        current_UPL += OrderProfit()+OrderSwap()+OrderCommission();
        sellTradeStatus = "Last SELL "+OrderSymbol()+" Opened at "+TimeToStr(OrderOpenTime(),TIME_DATE|TIME_SECONDS)+" P/L = "+DoubleToStr(OrderProfit()+OrderSwap()+OrderCommission(),2);
        TradeExist = true;
      }
    }
  }
  if (current_UPL!=0) combined_PL = "Combined P/L = "+DoubleToStr(current_UPL,2);
  if (current_UPL<max_DD) max_DD = current_UPL; // track worst DD
  return (TradeExist);
}

//+------------------------------------------------------------------+
void CloseTrades()
//+------------------------------------------------------------------+
{

  for (int i = 0; i < OrdersTotal(); i++)
  {
    OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
    if (OrderMagicNumber() == MagicNumber)
    {
      // Print(" order type "+OrderType() +" ticket "+ OrderTicket());
      if (OrderType() == OP_BUY)
      {
        Print("In full closure mode.  Closing a ", OrderSymbol(), " buy trade...");
        while(IsTradeContextBusy()) Sleep(100);
        RefreshRates();
        double buycloseprice = MarketInfo(OrderSymbol(), MODE_BID);
        string buySym = OrderSymbol();
        double buyProfit = OrderProfit()+OrderSwap()+OrderCommission();
        OrderClose(OrderTicket(), OrderLots(), buycloseprice, Slippage, White);
        i=0;//restart history loop to make sure we won't miss the other trade
      }
      else
      if (OrderType() == OP_SELL)
      {
        Print("In full closure mode.  Closing a ", OrderSymbol(), " sell trade...");
        while(IsTradeContextBusy()) Sleep(100);
        RefreshRates();
        double sellcloseprice = MarketInfo(OrderSymbol(), MODE_ASK);
        string sellSym = OrderSymbol();
        double sellProfit = OrderProfit()+OrderSwap()+OrderCommission();
        OrderClose(OrderTicket(), OrderLots(), sellcloseprice, Slippage, White);
        i=0;//restart history loop to make sure we won't miss the other trade
      }
    }
  }

  string comment = 
     TimeToStr(TimeCurrent(),TIME_SECONDS)
     +" StochDiff:Closed BUY "+buySym+"@"+DoubleToStr(buycloseprice,MarketInfo(buySym,MODE_DIGITS))+"="+buyProfit
     +" & SELL "+sellSym+"@"+DoubleToStr(sellcloseprice,MarketInfo(sellSym,MODE_DIGITS))
     +"="+DoubleToStr(buyProfit+sellProfit,2)+", max DD="+DoubleToStr(max_DD,2)
     ;
  Print(comment);
  drawVLine(comment,TimeCurrent(),Red);
}

//+------------------------------------------------------------------+
void wait_until_order_in_history(int ticket)
//+------------------------------------------------------------------+
{
  if (ticket<0) return;
  // wait until order is entered into the history
  int t=GetTickCount();
  while(!OrderSelect(ticket,SELECT_BY_TICKET)){}
  Print("Order entered into history in ",DoubleToStr(GetTickCount()-t,0),"ms");
}

//+------------------------------------------------------------------+
void placeOrders(string buySymbol, string sellSymbol)
//+------------------------------------------------------------------+
{
  string comment;

  // buy trade
  while(IsTradeContextBusy()) Sleep(100);
  RefreshRates();
  double buyPrice = MarketInfo(buySymbol, MODE_ASK);
  Print("Placing buy  order at " + buyPrice + " for " + buySymbol);
  comment = "StochDiff+"+Pair1+"-"+Pair2+" "+MagicNumber+" M"+Period();
  int ticketlong = OrderSend(buySymbol, OP_BUY, LotSize, NormalizeDouble(buyPrice, MarketInfo(buySymbol, MODE_DIGITS)), Slippage, 0, 0, comment, MagicNumber, 0, Blue);
  if (ticketlong < 0)
  {
    int check = GetLastError();
    if (check != 0) Print("StochDiff OP_BUY failed with error: ", ErrorDescription(check));
    return;
  }

  // sell trade
  while(IsTradeContextBusy()) Sleep(100);
  RefreshRates();
  double sellPrice = MarketInfo(sellSymbol, MODE_BID);
  Print("Placing sell  order at " + sellPrice + " for  " + sellSymbol);
  comment = "StochDiff+"+Pair1+"-"+Pair2+" "+MagicNumber+" M"+Period();
  int ticketshort = OrderSend(sellSymbol, OP_SELL, LotSize, NormalizeDouble(sellPrice, MarketInfo(sellSymbol, MODE_DIGITS)), Slippage, 0, 0, comment, MagicNumber, 0, Red);
  if (ticketshort < 0)
  {
    check = GetLastError();
    if (check != 0) Print("StochDiff OP_SELL failed with error: ", ErrorDescription(check));
    return;
  }

  wait_until_order_in_history(ticketlong);
  wait_until_order_in_history(ticketshort);

  OrderSelect(ticketlong,SELECT_BY_TICKET);  double buyprice = OrderOpenPrice();
  OrderSelect(ticketshort,SELECT_BY_TICKET); double sellprice = OrderOpenPrice();
  drawVLine("StochDiff:+"+buySymbol+"@"+DoubleToStr(buyprice,MarketInfo(buySymbol,MODE_DIGITS))
     +",-"+sellSymbol+"@"+DoubleToStr(sellprice,MarketInfo(sellSymbol,MODE_DIGITS))
     +" "+TimeToStr(TimeCurrent(),TIME_SECONDS), TimeCurrent(), Blue);
}

//+------------------------------------------------------------------
void drawVLine(string objname, int time, color c, int style=STYLE_SOLID, int width=0)
//+------------------------------------------------------------------
{
  ObjectCreate(objname, OBJ_VLINE, 0, time, 0);
  ObjectSet(objname, OBJPROP_COLOR, c);
  ObjectSet(objname, OBJPROP_STYLE, style);
}

//end

